@@ -20,7 +20,7 @@ use Mojo::RabbitMQ::Client::LocalQueue;
 require Mojo::RabbitMQ::Client::Consumer;
 require Mojo::RabbitMQ::Client::Publisher;
 
-our $VERSION = "0.1.0";
+our $VERSION = "0.2.0";
 
 use constant DEBUG => $ENV{MOJO_RABBITMQ_DEBUG} // 0;
 
@@ -273,80 +273,48 @@ sub _handle {
 
 sub _read {
   my ($self, $id, $chunk) = @_;
-  my $chunk_len = length($chunk);
-  my $buffer_len = defined $self->{buffer} ? length($self->{buffer}) : 0;
 
   warn "<- @{[dumper $chunk]}" if DEBUG;
-
-  if ($chunk_len + $buffer_len > $self->max_buffer_size) {
-    $self->{buffer} = '';
-    return;
-  }
-
-  my $data = $self->{buffer} .= $chunk;
-
-  if (length($data) < 8) {
-    return;
-  }
-
-  my ($type_id, $channel, $length,) = unpack 'CnN', substr $data, 0, 7;
-  if (!defined $type_id || !defined $channel || !defined $length) {
-    $self->{buffer} = '';
-    return;
-  }
-
-  if (length($data) < $length) {
-    return;
-  }
-
-  weaken $self;
-  $self->_loop->next_tick(sub { $self->_parse_frames });
+  $self->{buffer} .= $chunk;
+  $self->_parse_frames;
 
   return;
 }
 
 sub _parse_frames {
   my $self = shift;
-  my $data = $self->{buffer};
 
-  my ($type_id, $channel, $length,) = unpack 'CnN', substr $data, 0, 7;
+  for my $frame (Net::AMQP->parse_raw_frames(\$self->{buffer})) {
 
-  my $stack = substr $self->{buffer}, 0, $length + 8, '';
+    if ($frame->isa('Net::AMQP::Frame::Heartbeat')) {
+      $self->heartbeat_received(time());
+    }
+    elsif ($frame->isa('Net::AMQP::Frame::Method')
+      and $frame->method_frame->isa('Net::AMQP::Protocol::Connection::Close'))
+    {
+      $self->is_open(0);
 
-  my ($frame) = Net::AMQP->parse_raw_frames(\$stack);
-
-  if ($frame->isa('Net::AMQP::Frame::Heartbeat')) {
-    $self->heartbeat_received(time());
-  }
-  elsif ($frame->isa('Net::AMQP::Frame::Method')
-    and $frame->method_frame->isa('Net::AMQP::Protocol::Connection::Close'))
-  {
-    $self->is_open(0);
-
-    $self->_write_frame(Net::AMQP::Protocol::Connection::CloseOk->new());
-    $self->emit(disconnect => "Server side disconnection: "
-        . $frame->method_frame->{reply_text});
-  }
-  elsif ($frame->channel == 0) {
-    $self->queue->push($frame);
-  }
-  else {
-    my $channel = $self->channels->{$frame->channel};
-    if (defined $channel) {
-      $channel->_push_queue_or_consume($frame);
+      $self->_write_frame(Net::AMQP::Protocol::Connection::CloseOk->new());
+      $self->emit(disconnect => "Server side disconnection: "
+          . $frame->method_frame->{reply_text});
+    }
+    elsif ($frame->channel == 0) {
+      $self->queue->push($frame);
     }
     else {
-      $self->emit(
-        error => "Unknown channel id received: "
-          . ($frame->channel // '(undef)'),
-        $frame
-      );
+      my $channel = $self->channels->{$frame->channel};
+      if (defined $channel) {
+        $channel->_push_queue_or_consume($frame);
+      }
+      else {
+        $self->emit(
+          error => "Unknown channel id received: "
+            . ($frame->channel // '(undef)'),
+          $frame
+        );
+      }
     }
   }
-
-  weaken $self;
-  $self->_loop->next_tick(sub { $self->_parse_frames })
-    if length($self->{buffer}) >= 8;
 }
 
 sub _connect {
